@@ -82,6 +82,7 @@ change_table = Table(
     Column('pending_status', Boolean, index=True, nullable=False),
     Column('pending_status_message', Text),
     Column('last_seen', DateTime, index=True),
+    Column('outdated', Boolean, index=True, nullable=False),
     )
 change_conflict_table = Table(
     'change_conflict', metadata,
@@ -199,6 +200,25 @@ class Project(object):
         self.subscribed = subscribed
         self.description = description
 
+    def delete(self):
+        # This unusual delete method is to accomodate the
+        # self-referential many-to-many relationship from
+        # change_conflict.  With the default cascade configuration,
+        # the entry from the association table is deleted regardless
+        # of whether the change being deleted is change1 or change2.
+        # However, when both changes are deleted at once (only likely
+        # to happen when the entire project is deleted), SQLAlchemy
+        # cascades through both relationships and attempts to delete
+        # the entry twice.  Since we rarely delete projects, we add a
+        # special case here to delete a project's changes one at a
+        # time to avoid this situation.
+        session = Session.object_session(self)
+        for c in self.changes:
+            session.delete(c)
+            session.flush()
+            session.expire_all()
+        session.delete(self)
+
     def createChange(self, *args, **kw):
         session = Session.object_session(self)
         args = [self] + list(args)
@@ -257,7 +277,7 @@ class Change(object):
                  hidden=False, reviewed=False, starred=False, held=False,
                  pending_rebase=False, pending_topic=False,
                  pending_starred=False, pending_status=False,
-                 pending_status_message=None):
+                 pending_status_message=None, outdated=False):
         self.project_key = project.key
         self.account_key = owner.key
         self.id = id
@@ -278,6 +298,7 @@ class Change(object):
         self.pending_starred = pending_starred
         self.pending_status = pending_status
         self.pending_status_message = pending_status_message
+        self.outdated = outdated
 
     def getCategories(self):
         categories = set([label.category for label in self.labels])
@@ -860,14 +881,20 @@ class DatabaseSession(object):
     def getChanges(self, query, unreviewed=False, sort_by='number'):
         self.database.log.debug("Search query: %s sort: %s" % (query, sort_by))
         q = self.session().query(Change).filter(self.search.parse(query))
+        if not isinstance(sort_by, (list, tuple)):
+            sort_by = [sort_by]
         if unreviewed:
             q = q.filter(change_table.c.hidden==False, change_table.c.reviewed==False)
-        if sort_by == 'updated':
-            q = q.order_by(change_table.c.updated)
-        elif sort_by == 'last-seen':
-            q = q.order_by(change_table.c.last_seen)
-        else:
-            q = q.order_by(change_table.c.number)
+        for s in sort_by:
+            if s == 'updated':
+                q = q.order_by(change_table.c.updated)
+            elif s == 'last-seen':
+                q = q.order_by(change_table.c.last_seen)
+            elif s == 'number':
+                q = q.order_by(change_table.c.number)
+            elif s == 'project':
+                q = q.filter(project_table.c.key == change_table.c.project_key)
+                q = q.order_by(project_table.c.name)
         self.database.log.debug("Search SQL: %s" % q)
         try:
             return q.all()
@@ -933,6 +960,9 @@ class DatabaseSession(object):
     def getHeld(self):
         return self.session().query(Change).filter_by(held=True).all()
 
+    def getOutdated(self):
+        return self.session().query(Change).filter_by(outdated=True).all()
+
     def getPendingMessages(self):
         return self.session().query(Message).filter_by(pending=True).all()
 
