@@ -65,11 +65,18 @@ EDIT_COMMIT_MESSAGE = 'edit commit message'
 SUBMIT_CHANGE = 'submit change'
 SORT_BY_NUMBER = 'sort by number'
 SORT_BY_UPDATED = 'sort by updated'
+SORT_BY_LAST_SEEN = 'sort by last seen'
 SORT_BY_REVERSE = 'reverse the sort'
 # Project list screen:
 TOGGLE_LIST_REVIEWED = 'toggle list reviewed'
 TOGGLE_LIST_SUBSCRIBED = 'toggle list subscribed'
 TOGGLE_SUBSCRIBED = 'toggle subscribed'
+NEW_PROJECT_TOPIC = 'new project topic'
+DELETE_PROJECT_TOPIC = 'delete project topic'
+MOVE_PROJECT_TOPIC = 'move to project topic'
+COPY_PROJECT_TOPIC = 'copy to project topic'
+REMOVE_PROJECT_TOPIC = 'remove from project topic'
+RENAME_PROJECT_TOPIC = 'rename project topic'
 # Diff screens:
 SELECT_PATCHSETS = 'select patchsets'
 NEXT_SELECTABLE = 'next selectable'
@@ -122,13 +129,20 @@ DEFAULT_KEYMAP = {
     EDIT_TOPIC: 'ctrl t',
     EDIT_COMMIT_MESSAGE: 'ctrl d',
     SUBMIT_CHANGE: 'ctrl u',
-    SORT_BY_NUMBER: 'n',
-    SORT_BY_UPDATED: 'u',
-    SORT_BY_REVERSE: 'R',
+    SORT_BY_NUMBER: [['S', 'n']],
+    SORT_BY_UPDATED: [['S', 'u']],
+    SORT_BY_LAST_SEEN: [['S', 's']],
+    SORT_BY_REVERSE: [['S', 'r']],
 
     TOGGLE_LIST_REVIEWED: 'l',
     TOGGLE_LIST_SUBSCRIBED: 'L',
     TOGGLE_SUBSCRIBED: 's',
+    NEW_PROJECT_TOPIC: [['T', 'n']],
+    DELETE_PROJECT_TOPIC: [['T', 'delete']],
+    MOVE_PROJECT_TOPIC: [['T', 'm']],
+    COPY_PROJECT_TOPIC: [['T', 'c']],
+    REMOVE_PROJECT_TOPIC: [['T', 'D']],
+    RENAME_PROJECT_TOPIC: [['T', 'r']],
 
     SELECT_PATCHSETS: 'p',
     NEXT_SELECTABLE: 'tab',
@@ -136,6 +150,17 @@ DEFAULT_KEYMAP = {
     INTERACTIVE_SEARCH: 'ctrl s',
 }
 
+# Hi vi users!  Add more things here!  This overrides the default
+# keymap, so anything not defined here will just use what's defined
+# above.
+VI_KEYMAP = {
+    QUIT: [[':', 'q']],
+    CURSOR_LEFT: 'h',
+    CURSOR_DOWN: 'j',
+    CURSOR_UP: 'k',
+    CURSOR_RIGHT: 'l',
+}
+
 URWID_COMMANDS = frozenset((
     urwid.REDRAW_SCREEN,
     urwid.CURSOR_UP,
@@ -156,7 +181,7 @@ FORMAT_SUBS = (
     (re.compile('ctrl '), 'CTRL-'),
     (re.compile('meta '), 'META-'),
     (re.compile('f(\d+)'), 'F\\1'),
-    (re.compile('([a-z][a-z]+)'), lambda x: string.upper(x.group(1))),
+    (re.compile('([a-z][a-z]+)'), lambda x: x.group(1).upper()),
     )
 
 def formatKey(key):
@@ -225,6 +250,25 @@ class KeyMap(object):
             ret.append(FURTHER_INPUT)
         return ret
 
+    def getFurtherCommands(self, keys):
+        if not keys:
+            return []
+        tree = self.keytree
+        for key in keys:
+            tree = tree.keys.get(key)
+            if not tree:
+                return []
+        return self._getFurtherCommands('', tree)
+
+    def _getFurtherCommands(self, keys, tree):
+        if keys:
+            ret = [(formatKey(keys), tree.commands[:])]
+        else:
+            ret = []
+        for subtree in tree.keys.values():
+            ret.extend(self._getFurtherCommands(keys + subtree.key, subtree))
+        return ret
+
     def getKeys(self, command):
         return self.commandmap.get(command, [])
 
