@@ -128,8 +128,23 @@ class MyEdit(urwid.Edit):
             self.ring.kill(text)
         return super(MyEdit, self).keypress(size, key)
 
+class LineBoxTitlePropertyMixin(object):
+
+    @property
+    def title(self):
+        return self._w.title_widget.text.strip()
+
+    @title.setter
+    def title(self, text):
+        return self._w.set_title(text)
+
+class SystemMessage(urwid.WidgetWrap, LineBoxTitlePropertyMixin):
+    def __init__(self, message):
+        w = urwid.Filler(urwid.Text(message, align='center'))
+        super(SystemMessage, self).__init__(urwid.LineBox(w, u'System Message'))
+
 @mouse_scroll_decorator.ScrollByWheel
-class ButtonDialog(urwid.WidgetWrap):
+class ButtonDialog(urwid.WidgetWrap, LineBoxTitlePropertyMixin):
     def __init__(self, title, message, entry_prompt=None,
                  entry_text='', buttons=[], ring=None):
         button_widgets = []
@@ -148,7 +163,34 @@ class ButtonDialog(urwid.WidgetWrap):
         listbox = urwid.ListBox(rows)
         super(ButtonDialog, self).__init__(urwid.LineBox(listbox, title))
 
-class TextEditDialog(urwid.WidgetWrap):
+class LineEditDialog(ButtonDialog):
+    signals = ['save', 'cancel']
+    def __init__(self, app, title, message, entry_prompt=None,
+                 entry_text='', ring=None):
+        self.app = app
+        save_button = FixedButton('Save')
+        cancel_button = FixedButton('Cancel')
+        urwid.connect_signal(save_button, 'click',
+                             lambda button:self._emit('save'))
+        urwid.connect_signal(cancel_button, 'click',
+                             lambda button:self._emit('cancel'))
+        super(LineEditDialog, self).__init__(title, message, entry_prompt,
+                                             entry_text,
+                                             buttons=[save_button,
+                                                      cancel_button],
+                                             ring=ring)
+
+    def keypress(self, size, key):
+        if not self.app.input_buffer:
+            key = super(LineEditDialog, self).keypress(size, key)
+        keys = self.app.input_buffer + [key]
+        commands = self.app.config.keymap.getCommands(keys)
+        if keymap.ACTIVATE in commands:
+            self._emit('save')
+            return None
+        return key
+
+class TextEditDialog(urwid.WidgetWrap, LineBoxTitlePropertyMixin):
     signals = ['save', 'cancel']
     def __init__(self, title, prompt, button, text, ring=None):
         save_button = FixedButton(button)
@@ -248,6 +290,62 @@ class SearchableText(urwid.Text):
         self._invalidate()
         return found
 
+class Searchable(object):
+    def searchInit(self):
+        self.search = None
+        self.results = []
+        self.current_result = 0
+
+    def searchValidChar(self, ch):
+        return urwid.util.is_wide_char(ch, 0) or (len(ch) == 1 and ord(ch) >= 32)
+
+    def searchKeypress(self, size, key):
+        if self.search is not None:
+            if self.searchValidChar(key) or key == 'backspace':
+                if key == 'backspace':
+                    self.search = self.search[:-1]
+                else:
+                    self.search += key
+                self.interactiveSearch(self.search)
+                return True
+            else:
+                commands = self.app.config.keymap.getCommands([key])
+                if keymap.INTERACTIVE_SEARCH in commands:
+                    self.nextSearchResult()
+                    return True
+                else:
+                    self.app.status.update(title=self.title)
+                    if not self.search:
+                        self.interactiveSearch(None)
+                    self.search = None
+                    if key in ['enter', 'esc']:
+                        return True
+        return False
+
+    def searchStart(self):
+        self.search = ''
+        self.app.status.update(title=("Search: "))
+
+    def interactiveSearch(self, search):
+        if search is not None:
+            self.app.status.update(title=("Search: " + search))
+        self.results = []
+        self.current_result = 0
+        for i, line in enumerate(self.listbox.body):
+            if hasattr(line, 'search'):
+                if line.search(search, 'search-result'):
+                    self.results.append(i)
+
+    def nextSearchResult(self):
+        if not self.results:
+            return
+        dest = self.results[self.current_result]
+        self.listbox.set_focus(dest)
+        self.listbox._invalidate()
+        self.current_result += 1
+        if self.current_result >= len(self.results):
+            self.current_result = 0
+
 class HyperText(urwid.Text):
     _selectable = True
 
