@@ -67,18 +67,18 @@ class SearchCompiler(object):
 if __name__ == '__main__':
     class Dummy(object):
         pass
-    query = 'status:open limit:50 age:2months'
+    query = 'recentlyseen:24 hours'
     lexer = tokenizer.SearchTokenizer()
     lexer.input(query)
     while True:
         token = lexer.token()
         if not token:
             break
-        print token
+        print(token)
 
     app = Dummy()
     app.config = Dummy()
     app.config.username = 'bob'
     search = SearchCompiler(app.config.username)
     x = search.parse(query)
-    print x
+    print(x)
