@@ -23,6 +23,7 @@ import threading
 import json
 import time
 import datetime
+import warnings
 
 import dateutil.parser
 try:
@@ -236,7 +237,8 @@ class SyncProjectListTask(Task):
             local_keys = set(local.keys())
 
             for name in local_keys-remote_keys:
-                session.delete(local[name])
+                self.log.info("Deleted project %s", name)
+                local[name].delete()
 
             for name in remote_keys-local_keys:
                 p = remote[name]
@@ -528,6 +530,24 @@ class SyncChangeByNumberTask(Task):
             sync.submitTask(task)
             self.log.debug("Sync change %s because it is number %s" % (c['id'], self.number))
 
+class SyncOutdatedChangesTask(Task):
+    def __init__(self, priority=NORMAL_PRIORITY):
+        super(SyncOutdatedChangesTask, self).__init__(priority)
+
+    def __eq__(self, other):
+        if other.__class__ == self.__class__:
+            return True
+        return False
+
+    def __repr__(self):
+        return '<SyncOutdatedChangesTask>'
+
+    def run(self, sync):
+        with sync.app.db.getSession() as session:
+            for change in session.getOutdated():
+                self.log.debug("Sync outdated change %s" % (change.id,))
+                sync.submitTask(SyncChangeTask(change.id, priority=self.priority))
+
 class SyncChangeTask(Task):
     def __init__(self, change_id, force_fetch=False, priority=NORMAL_PRIORITY):
         super(SyncChangeTask, self).__init__(priority)
@@ -546,14 +566,37 @@ class SyncChangeTask(Task):
 
     def run(self, sync):
         start_time = time.time()
+        try:
+            self._syncChange(sync)
+            end_time = time.time()
+            total_time = end_time - start_time
+            self.log.info("Synced change %s in %0.5f seconds.", self.change_id, total_time)
+        except Exception:
+            try:
+                self.log.error("Marking change %s outdated" % (self.change_id,))
+                with sync.app.db.getSession() as session:
+                    change = session.getChangeByID(self.change_id)
+                    if change:
+                        change.outdated = True
+            except Exception:
+                self.log.exception("Error while marking change %s as outdated" % (self.change_id,))
+            raise
+
+    def _syncChange(self, sync):
         app = sync.app
         remote_change = sync.get('changes/%s?o=DETAILED_LABELS&o=ALL_REVISIONS&o=ALL_COMMITS&o=MESSAGES&o=DETAILED_ACCOUNTS&o=CURRENT_ACTIONS&o=ALL_FILES' % self.change_id)
         # Perform subqueries this task will need outside of the db session
         for remote_commit, remote_revision in remote_change.get('revisions', {}).items():
             remote_comments_data = sync.get('changes/%s/revisions/%s/comments' % (self.change_id, remote_commit))
             remote_revision['_gertty_remote_comments_data'] = remote_comments_data
-        remote_conflicts = sync.query(['q=status:open+is:mergeable+conflicts:%s' %
-                                       remote_change['_number']])
+        try:
+            remote_conflicts = sync.query(['q=status:open+is:mergeable+conflicts:%s' %
+                                           remote_change['_number']])
+        except Exception:
+            self.log.exception("Unable to sync conflicts for change %s" % self.change_id)
+            warnings.warn("Unable to sync conflicts for change %s" % self.change_id)
+            remote_conflicts = []
+
         fetches = collections.defaultdict(list)
         parent_commits = set()
         with app.db.getSession() as session:
@@ -564,6 +607,18 @@ class SyncChangeTask(Task):
                                              email=remote_change['owner'].get('email'))
             if not change:
                 project = session.getProjectByName(remote_change['project'])
+                if not project:
+                    self.log.debug("Project %s unknown while syncing change" % (
+                        remote_change['project'],))
+                    remote_project = sync.get('projects/%s' %
+                                              (urlparse.quote_plus(remote_change['project']),))
+                    if remote_project:
+                        project = session.createProject(
+                            remote_project['name'],
+                            description=remote_project.get('description', ''))
+                        self.log.info("Created project %s", project.name)
+                        self.results.append(ProjectAddedEvent(project))
+                        sync.submitTask(SyncProjectBranchesTask(project.name, self.priority))
                 created = dateutil.parser.parse(remote_change['created'])
                 updated = dateutil.parser.parse(remote_change['updated'])
                 change = project.createChange(remote_change['id'], account, remote_change['_number'],
@@ -858,6 +913,7 @@ class SyncChangeTask(Task):
                         change.reviewed = False
                         result.review_flag_changed = True
                         app.project_cache.clear(change.project)
+            change.outdated = False
         for url, refs in fetches.items():
             self.log.debug("Fetching from %s with refs %s", url, refs)
             try:
@@ -869,9 +925,6 @@ class SyncChangeTask(Task):
                 for ref in refs:
                     self.log.debug("git fetch %s %s" % (url, ref))
                     repo.fetch(url, ref)
-        end_time = time.time()
-        total_time = end_time - start_time
-        self.log.info("Synced change %s in %0.5f seconds.", self.change_id, total_time)
 
 class CheckReposTask(Task):
     # on startup, check all projects
@@ -898,16 +951,21 @@ class CheckReposTask(Task):
                 except gitrepo.GitCloneError:
                     missing = True
                 if missing or app.fetch_missing_refs:
-                    sync.submitTask(CheckRevisionsTask(project.key,
-                                                       priority=LOW_PRIORITY))
+                    sync.submitTask(
+                        CheckRevisionsTask(project.key,
+                                           force_fetch=app.fetch_missing_refs,
+                                           priority=LOW_PRIORITY)
+                    )
             except Exception:
                 self.log.exception("Exception checking repo %s" %
                                    (project.name,))
 
 class CheckRevisionsTask(Task):
-    def __init__(self, project_key, priority=NORMAL_PRIORITY):
+    def __init__(self, project_key, force_fetch=False,
+                 priority=NORMAL_PRIORITY):
         super(CheckRevisionsTask, self).__init__(priority)
         self.project_key = project_key
+        self.force_fetch = force_fetch
 
     def __repr__(self):
         return '<CheckRevisionsTask %s>' % (self.project_key,)
@@ -937,7 +995,9 @@ class CheckRevisionsTask(Task):
                 else:
                     to_sync.add(change.id)
         for change_id in to_sync:
-            sync.submitTask(SyncChangeTask(change_id, priority=self.priority))
+            sync.submitTask(SyncChangeTask(change_id,
+                                           force_fetch=self.force_fetch,
+                                           priority=self.priority))
 
 class UploadReviewsTask(Task):
     def __repr__(self):
@@ -1328,6 +1388,7 @@ class Sync(object):
             self.submitTask(SyncProjectListTask(HIGH_PRIORITY))
             self.submitTask(SyncSubscribedProjectsTask(NORMAL_PRIORITY))
             self.submitTask(SyncSubscribedProjectBranchesTask(LOW_PRIORITY))
+            self.submitTask(SyncOutdatedChangesTask(LOW_PRIORITY))
             self.submitTask(PruneDatabaseTask(self.app.config.expire_age, LOW_PRIORITY))
             self.periodic_thread = threading.Thread(target=self.periodicSync)
             self.periodic_thread.daemon = True
@@ -1343,6 +1404,7 @@ class Sync(object):
                 if now-hourly > 3600:
                     hourly = now
                     self.pruneDatabase()
+                    self.syncOutdatedChanges()
             except Exception:
                 self.log.exception('Exception in periodicSync')
 
@@ -1366,7 +1428,10 @@ class Sync(object):
             task.run(self)
             task.complete(True)
             self.queue.complete(task)
-        except (requests.ConnectionError, OfflineError) as e:
+        except (requests.ConnectionError, OfflineError,
+                requests.exceptions.ChunkedEncodingError,
+                requests.exceptions.ReadTimeout
+        ) as e:
             self.log.warning("Offline due to: %s" % (e,))
             if not self.offline:
                 self.submitTask(GetVersionTask(HIGH_PRIORITY))
@@ -1472,6 +1537,13 @@ class Sync(object):
             for subtask in task.tasks:
                 subtask.wait()
 
+    def syncOutdatedChanges(self):
+        task = SyncOutdatedChangesTask(LOW_PRIORITY)
+        self.submitTask(task)
+        if task.wait():
+            for subtask in task.tasks:
+                subtask.wait()
+
     def _syncChangeByCommit(self, commit, priority):
         # Accumulate sync change by commit tasks because they often
         # come in batches.  This method assumes it is being called
