@@ -47,6 +47,7 @@ COLUMNS = [
     ColumnInfo('Topic',   'weight',  1),
     ColumnInfo('Owner',   'weight',  1),
     ColumnInfo('Updated', 'given',  10),
+    ColumnInfo('Size',    'given',   4),
 ]
 
 
@@ -99,6 +100,10 @@ class ChangeRow(urwid.Button, ChangeListColumns):
                         'negative-label': 'focused-negative-label',
                         'min-label': 'focused-min-label',
                         'max-label': 'focused-max-label',
+
+
+                        'added-graph': 'focused-added-graph',
+                        'removed-graph': 'focused-removed-graph',
                         }
 
     def selectable(self):
@@ -114,6 +119,7 @@ class ChangeRow(urwid.Button, ChangeListColumns):
         self.subject = mywid.SearchableText(u'', wrap='clip')
         self.number = mywid.SearchableText(u'')
         self.updated = mywid.SearchableText(u'')
+        self.size = mywid.SearchableText(u'', align='right')
         self.project = mywid.SearchableText(u'', wrap='clip')
         self.owner = mywid.SearchableText(u'', wrap='clip')
         self.branch = mywid.SearchableText(u'', wrap='clip')
@@ -142,6 +148,35 @@ class ChangeRow(urwid.Button, ChangeListColumns):
             return True
         return False
 
+    def _makeSize(self, added, removed):
+        # Removed is a red graph on top, added is a green graph on bottom.
+        #
+        # The graph is 4 cells wide.  If both the red and green graphs
+        # are in the cell, we set the bg to red, fg to green, and set
+        # a box in the bottom half of the cell.
+        #
+        # If only one of the graphs is in the cell, we set a box in
+        # either the top or bottom of the cell, and set the fg color
+        # appropriately.  This is so that the reverse-video which
+        # operates on the line when focused works as expected.
+
+        lower_box = u'\u2584'
+        upper_box = u'\u2580'
+        ret = []
+        # The graph is logarithmic -- one cell for each order of
+        # magnitude.
+        for threshold in [1, 10, 100, 1000]:
+            color = []
+            if (added > threshold and removed > threshold):
+                ret.append(('added-removed-graph', lower_box))
+            elif (added > threshold):
+                ret.append(('added-graph', lower_box))
+            elif (removed > threshold):
+                ret.append(('removed-graph', upper_box))
+            else:
+                ret.append(' ')
+        return ret
+
     def update(self, change, categories):
         if change.reviewed or change.hidden:
             style = 'reviewed-change'
@@ -175,6 +210,14 @@ class ChangeRow(urwid.Button, ChangeListColumns):
             self.updated.set_text(updated_time.strftime("%I:%M %p").upper())
         else:
             self.updated.set_text(updated_time.strftime("%Y-%m-%d"))
+        total_added = 0
+        total_removed = 0
+        for rfile in change.revisions[-1].files:
+            if rfile.status is None:
+                continue
+            total_added += rfile.inserted or 0
+            total_removed += rfile.deleted or 0
+        self.size.set_text(self._makeSize(total_added, total_removed))
 
         self.category_columns = []
         for category in categories:
@@ -204,6 +247,7 @@ class ChangeListHeader(urwid.WidgetWrap, ChangeListColumns):
         self.subject = urwid.Text(u'Subject', wrap='clip')
         self.number = urwid.Text(u'Number')
         self.updated = urwid.Text(u'Updated')
+        self.size = urwid.Text(u'Size')
         self.project = urwid.Text(u'Project', wrap='clip')
         self.owner = urwid.Text(u'Owner', wrap='clip')
         self.branch = urwid.Text(u'Branch', wrap='clip')
@@ -223,7 +267,7 @@ class ChangeListHeader(urwid.WidgetWrap, ChangeListColumns):
 @mouse_scroll_decorator.ScrollByWheel
 class ChangeListView(urwid.WidgetWrap, mywid.Searchable):
     required_columns = set(['Number', 'Subject', 'Updated'])
-    optional_columns = set(['Topic', 'Branch'])
+    optional_columns = set(['Topic', 'Branch', 'Size'])
 
     def getCommands(self):
         if self.project_key:
@@ -335,9 +379,11 @@ class ChangeListView(urwid.WidgetWrap, mywid.Searchable):
             change_list = session.getChanges(self.query, self.unreviewed,
                                              sort_by=self.sort_by)
             if self.unreviewed:
-                self.title = u'Unreviewed changes in %s' % self.query_desc
+                self.title = (u'Unreviewed %d changes in %s' %
+                    (len(change_list), self.query_desc))
             else:
-                self.title = u'All changes in %s' % self.query_desc
+                self.title = (u'All %d changes in %s' %
+                    (len(change_list), self.query_desc))
             self.short_title = self.query_desc
             if '/' in self.short_title and ' ' not in self.short_title:
                 i = self.short_title.rfind('/')
