@@ -24,7 +24,7 @@ from gertty import gitrepo
 from gertty import sync
 from gertty.view import mouse_scroll_decorator
 
-class PatchsetDialog(urwid.WidgetWrap):
+class PatchsetDialog(urwid.WidgetWrap, mywid.LineBoxTitlePropertyMixin):
     signals = ['ok', 'cancel']
 
     def __init__(self, patchsets, old, new):
@@ -156,18 +156,22 @@ class DiffContextButton(urwid.WidgetWrap):
         self.view.expandChunk(self.diff, self.chunk, from_end=-10)
 
 @mouse_scroll_decorator.ScrollByWheel
-class BaseDiffView(urwid.WidgetWrap):
-    def help(self):
-        key = self.app.config.keymap.formatKeys
+class BaseDiffView(urwid.WidgetWrap, mywid.Searchable):
+    def getCommands(self):
         return [
-            (key(keymap.ACTIVATE),
+            (keymap.ACTIVATE,
              "Add an inline comment"),
-            (key(keymap.SELECT_PATCHSETS),
+            (keymap.SELECT_PATCHSETS,
              "Select old/new patchsets to diff"),
-            (key(keymap.INTERACTIVE_SEARCH),
+            (keymap.INTERACTIVE_SEARCH,
              "Interactive search"),
             ]
 
+    def help(self):
+        key = self.app.config.keymap.formatKeys
+        commands = self.getCommands()
+        return [(c[0], key(c[0]), c[1]) for c in commands]
+
     def __init__(self, app, new_revision_key):
         super(BaseDiffView, self).__init__(urwid.Pile([]))
         self.log = logging.getLogger('gertty.view.diff')
@@ -178,9 +182,7 @@ class BaseDiffView(urwid.WidgetWrap):
 
     def _init(self):
         del self._w.contents[:]
-        self.search = None
-        self.results = []
-        self.current_result = None
+        self.searchInit()
         with self.app.db.getSession() as session:
             new_revision = session.getRevision(self.new_revision_key)
             old_comments = []
@@ -214,6 +216,7 @@ class BaseDiffView(urwid.WidgetWrap):
                 new_revision.change.project.name,
                 new_revision.change.number,
                 old_str, new_revision.number)
+            self.short_title = u'Diff of %s' % (new_revision.change.number,)
             self.new_revision_num = new_revision.number
             self.change_key = new_revision.change.key
             self.project_name = new_revision.change.project.name
@@ -303,6 +306,7 @@ class BaseDiffView(urwid.WidgetWrap):
                         lines += self.makeLines(diff, chunk.lines[-10:], comment_lists)
                         del chunk.lines[-10:]
                     chunk.calcRange()
+                    chunk.button.update()
                     if not chunk.lines:
                         lines.remove(button)
                 else:
@@ -352,7 +356,7 @@ class BaseDiffView(urwid.WidgetWrap):
                     if from_start:
                         self.expandChunk(diff, chunk, comment_lists, from_start=i+10)
                     else:
-                        self.expandChunk(diff, chunk, comment_lists, from_end=i-10)
+                        self.expandChunk(diff, chunk, comment_lists, from_end=0-(len(chunk.lines)-i)-10)
                     break
 
     def expandChunk(self, diff, chunk, comment_lists={}, from_start=None, from_end=None,
@@ -438,30 +442,9 @@ class BaseDiffView(urwid.WidgetWrap):
             context = item.context
         return context
 
-    def search_valid_char(self, ch):
-        return urwid.util.is_wide_char(ch, 0) or (len(ch) == 1 and ord(ch) >= 32)
-
     def keypress(self, size, key):
-        if self.search is not None:
-            if self.search_valid_char(key) or key == 'backspace':
-                if key == 'backspace':
-                    self.search = self.search[:-1]
-                else:
-                    self.search += key
-                self.interactiveSearch(self.search)
-                return None
-            else:
-                commands = self.app.config.keymap.getCommands([key])
-                if keymap.INTERACTIVE_SEARCH in commands:
-                    self.nextSearchResult()
-                    return None
-                else:
-                    self.app.status.update(title=self.title)
-                    if not self.search:
-                        self.interactiveSearch(None)
-                    self.search = None
-                    if key in ['enter', 'esc']:
-                        return None
+        if self.searchKeypress(size, key):
+            return None
 
         old_focus = self.listbox.focus
         if not self.app.input_buffer:
@@ -484,8 +467,7 @@ class BaseDiffView(urwid.WidgetWrap):
             self.openPatchsetDialog()
             return None
         if keymap.INTERACTIVE_SEARCH in commands:
-            self.search = ''
-            self.app.status.update(title=("Search: "))
+            self.searchStart()
             return None
         return key
 
@@ -556,23 +538,3 @@ class BaseDiffView(urwid.WidgetWrap):
         self.app.backScreen()
         self.old_revision_key, self.new_revision_key = dialog.getSelected()
         self._init()
-
-    def interactiveSearch(self, search):
-        if search is not None:
-            self.app.status.update(title=("Search: " + search))
-        self.results = []
-        self.current_result = 0
-        for i, line in enumerate(self.listbox.body):
-            if hasattr(line, 'search'):
-                if line.search(search, 'search-result'):
-                    self.results.append(i)
-
-    def nextSearchResult(self):
-        if not self.results:
-            return
-        dest = self.results[self.current_result]
-        self.listbox.set_focus(dest)
-        self.listbox._invalidate()
-        self.current_result += 1
-        if self.current_result >= len(self.results):
-            self.current_result = 0
